﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace AirplaneWithRadar
{
    public partial class FormMap : Form
    {
        private AbstractMap _abstractMap;
        public FormMap()
        {
            InitializeComponent();
            _abstractMap = new SimpleMap();
        }
        /// <summary>
        /// Заполнение информации по объекту
        /// </summary>
        /// <param name="airplane"></param>
        private void SetData(DrawningAirplane airplane)
        {
            toolStripStatusLabelSpeed.Text = $"Скорость: {airplane.Airplane.Speed}";
            toolStripStatusLabelWeight.Text = $"Вес: {airplane.Airplane.Weight}";
            toolStripStatusLabelBodyColor.Text = $"Цвет: {airplane.Airplane.BodyColor.Name}";
            pictureBoxAirplane.Image = _abstractMap.CreateMap(pictureBoxAirplane.Width, pictureBoxAirplane.Height, new DrawningObjectAirplane(airplane));
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            IAirplanePortholes typeAirplanePortholes = new DrawningAirplanePortholes();
            switch (comboBoxTypePortholes.Text)
            {
                case "Треугольные":
                    typeAirplanePortholes = new DrawningDeltaPortholes();
                    break;
                case "Ромбовидные":
                    typeAirplanePortholes = new DrawningRhombusPortholes();
                    break;
            }
            Random rnd = new();
            var airplane = new DrawningAirplane(rnd.Next(100, 300), rnd.Next(1000, 2000), Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)), typeAirplanePortholes);
            airplane.DrawningPortholes.CountPortholes = Convert.ToInt32(comboBoxPortholes.Text);
            SetData(airplane);
        }

        /// <summary>
        /// Изменение размеров формы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBoxAirplane.Image = _abstractMap?.MoveObject(dir);
        }

        /// <summary>
        /// Обработка нажатия кнопки "Модификация"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateModif_Click(object sender, EventArgs e)
        {
            IAirplanePortholes typeAirplanePortholes = new DrawningAirplanePortholes();
            switch (comboBoxTypePortholes.Text)
            {
                case "Треугольные":
                    typeAirplanePortholes = new DrawningDeltaPortholes();
                    break;
                case "Ромбовидные":
                    typeAirplanePortholes = new DrawningRhombusPortholes();
                    break;
            }
            Random rnd = new();
            var airplane = new DrawningAirplaneWithRadar(rnd.Next(100, 300), rnd.Next(1000, 2000),
            Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
            Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
            Convert.ToBoolean(rnd.Next(0, 2)),
            Convert.ToBoolean(rnd.Next(0, 2)),
            typeAirplanePortholes);
            airplane.DrawningPortholes.CountPortholes = Convert.ToInt32(comboBoxPortholes.Text);
            SetData(airplane);
        }

        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    _abstractMap = new SimpleMap();
                    break;
                case "Моя карта":
                    _abstractMap = new MyMap();
                    break;
            }
        }
    }
}
