﻿using AirBomber;

namespace AirplaneWithRadar
{
    public partial class FormMapWithSetAirplanes : Form
    {
        /// <summary>
        /// Объект от класса карты с набором объектов
        /// </summary>
        private MapWithSetAirplanesGeneric<DrawningObjectAirplane, AbstractMap> _mapAirplanesCollectionGeneric;
        private GeneratorAirplane<EntityAirplane, IAirplanePortholes> _generatorAirplane;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMapWithSetAirplanes()
        {
            _generatorAirplane = new(100, 100);
            InitializeComponent();
        }
        /// <summary>
        /// Добавление самолета на карту
        /// </summary>
        /// <param name="airplane">самолет.</param>
        private void AddAirplaneInMap(DrawningObjectAirplane airplane)
        {
            if (airplane == null || (_mapAirplanesCollectionGeneric + airplane) == -1)
            {
                MessageBox.Show("Не удалось добавить объект");
            }
            else
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _mapAirplanesCollectionGeneric.ShowSet();
            }
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    map = new SimpleMap();
                    break;
                case "Моя карта":
                    map = new MyMap();
                    break;
            }
            if (map != null)
            {
                _mapAirplanesCollectionGeneric = new MapWithSetAirplanesGeneric<DrawningObjectAirplane, AbstractMap>(pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapAirplanesCollectionGeneric = null;
            }
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapAirplanesCollectionGeneric?.MoveObject(dir) ?? pictureBox.Image;
        }
        /// <summary>
        /// Добавления сущности в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddTypeOfEntity_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            Color colorBody = Color.FromArgb(rnd.Next() % 256, rnd.Next() % 256, rnd.Next() % 256);
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                colorBody = dialog.Color;
            }
            var entity = new EntityAirplane((int)numericSpeed.Value, (int)numericWeight.Value, colorBody);
            _generatorAirplane.AddTypeOfEntity(entity);
            MessageBox.Show($"Добавлены свойства самолета:\n" +
                $"Вес: {entity.Weight}\n" +
                $"Скорость: {entity.Speed}\n" +
                $"Цвет: {colorBody.Name}",
                "Успешно добавлены свойства");
        }
        /// <summary>
        /// Добавления типа иллюминаторов и их колличетсва в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddTypeOfPortholes_Click(object sender, EventArgs e)
        {
            IAirplanePortholes? typeAirplanePortholes = null;
            switch (comboTypePotholes.Text)
            {
                case "Треугольные":
                    typeAirplanePortholes = new DrawningDeltaPortholes();
                    break;
                case "Ромбовидные":
                    typeAirplanePortholes = new DrawningRhombusPortholes();
                    break;
                default:
                    typeAirplanePortholes = new DrawningAirplanePortholes();
                    break;
            }
            typeAirplanePortholes.CountPortholes = Convert.ToInt32(comboBoxPortholes.Text);
            _generatorAirplane.AddTypeOfPortholes(typeAirplanePortholes);
        }
        /// <summary>
        /// Генерация самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGenerateAirplane_Click(object sender, EventArgs e)
        {
            if (_mapAirplanesCollectionGeneric == null)
            {
                return;
            }
            var airplane = _generatorAirplane.Generate();
            if (airplane == null)
            {
                MessageBox.Show("Не удалось сгенерировать самолет. Добавьте свойства для генерации"
                    , "Генерация самолета");
                return;
            }
            AddAirplaneInMap(airplane);

        }
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddAirplane_Click(object sender, EventArgs e)
        {
            if (_mapAirplanesCollectionGeneric == null)
            {
                return;
            }
            FormAirplane form = new();
            if (form.ShowDialog() == DialogResult.OK && form.SelectedAirplane != null)
            {
                AddAirplaneInMap(new(form.SelectedAirplane));
            }
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveAirplane_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_mapAirplanesCollectionGeneric - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _mapAirplanesCollectionGeneric.ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (_mapAirplanesCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapAirplanesCollectionGeneric.ShowSet();
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (_mapAirplanesCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapAirplanesCollectionGeneric.ShowOnMap();
        }
    }
}
