﻿using AirplaneWithRadar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    /// <summary>
    /// Класс, который генирирует самолет из разнообразного количества сущностей и типа иллюминаторов
    /// </summary>
    /// <typeparam name="T">Класс Сущность самолет</typeparam>
    /// <typeparam name="U">Класс иллюминатора самолета</typeparam>
    internal class GeneratorAirplane<T, U>
        where T : EntityAirplane
        where U : class, IAirplanePortholes
    {
        private readonly T[] typesOfEntity;
        private readonly U[] typesOfPortholes;

        public int NumTypesOfEntity { get; private set; }
        public int NumTypesOfPortholes { get; private set; }

        public GeneratorAirplane(int countTypesOfEntity, int countTypesOfPortholes)
        {
            typesOfEntity = new T[countTypesOfEntity];
            typesOfPortholes = new U[countTypesOfPortholes];
        }
        /// <summary>
        /// Добавляет возможный тип сущности при генерации самолета
        /// </summary>
        /// <param name="type">тип</param>
        /// <returns>Успешно ли проведена операция</returns>
        public virtual bool AddTypeOfEntity(T type)
        {
            if (NumTypesOfEntity >= typesOfEntity.Length)
            {
                return false;
            }
            typesOfEntity[NumTypesOfEntity++] = type;
            return true;
        }
        /// <summary>
        /// Добавляет возможный тип иллюминатора при генерации самолета
        /// </summary>
        /// <param name="type">тип</param>
        /// <returns>Успешно ли проведена операция</returns>
        public virtual bool AddTypeOfPortholes(U type)
        {
            if (NumTypesOfPortholes >= typesOfPortholes.Length)
            {
                return false;
            }
            typesOfPortholes[NumTypesOfPortholes++] = type;
            return true;
        }
        /// <summary>
        /// Генерирует объект отрисовки
        /// </summary>
        /// <returns>Возвращает объект отрисовки, либо null, если не были добавлены типы для выборки</returns>
        public DrawningObjectAirplane? Generate()
        {
            if (NumTypesOfPortholes == 0 || NumTypesOfEntity == 0)
            {
                return null;
            }
            var rnd = new Random();
            var airplane = new DrawningAirplane(typesOfEntity[rnd.Next() % NumTypesOfEntity], typesOfPortholes[rnd.Next() % NumTypesOfPortholes]);
            return new DrawningObjectAirplane(airplane);
        }
    }
}
